;****************************************************************************
;*
;*                        The Universal VESA VBE
;*
;*                  Copyright (C) 1993 Kendall Bennett.
;*                          All rights reserved.
;*
;* Filename:    $RCSfile: _acumos.asm $
;* Version:     $Revision: 1.1 $
;*
;* Language:    8086 Assembler
;* Environment: IBM PC (MS DOS)
;*
;* Description: Assembly language support routines for AcuMos SuperVGA's.
;*
;*              Contains code to perform initialisation, bank switching
;*              and extended page flipping if possible.
;*
;* $Id: _acumos.asm 1.1 1993/09/19 01:26:40 kjb Exp $
;*
;* Revision History:
;* -----------------
;*
;* $Log: _acumos.asm $
;* Revision 1.1  1993/09/19  01:26:40  kjb
;* Initial revision
;*
;****************************************************************************

;----------------------------------------------------------------------------
; ACUMOS_bank   Change to a new 64k bank (reading and writing)
;----------------------------------------------------------------------------
;
; Entry:        AX  - Number of 64k bank
;
; Registers:    All preserved!
;
;----------------------------------------------------------------------------
PROC    ACUMOS_bank far

        cli
        push    ax
        push    dx
        mov     dx,3CEh
        shl     al,4                ; Convert to 4k bank number
        mov     ah,al
        mov     al,9
        out     dx,ax
        pop     dx
        pop     ax
        sti
        ret

ENDP    ACUMOS_bank

ACUMOS_bank_len     =   ($-ACUMOS_bank)

;----------------------------------------------------------------------------
; ACUMOS_setup  Setup the SuperVGA for correct operation
;----------------------------------------------------------------------------
PROC    ACUMOS_setup    near

        wrinx   3C4h, 6, 12h        ; Enable extended registers
        ret

ENDP    ACUMOS_setup

ACUMOS_setup_len    =   ($-ACUMOS_setup)

;----------------------------------------------------------------------------
; ACUMOS_exit   Return the SuperVGA to default operation
;----------------------------------------------------------------------------
PROC    ACUMOS_exit near

        wrinx   3C4h, 6, 0          ; Disable extended registers
        ret

ENDP    ACUMOS_exit

ACUMOS_exit_len =   ($-ACUMOS_exit)

;----------------------------------------------------------------------------
; ACUMOS_init
;----------------------------------------------------------------------------
; We have a ACUMOS SuperVGA, so setup the appropriate vectors.
;----------------------------------------------------------------------------
PROC    ACUMOS_init

        mov     [TwoBanks],false
        mov     [WriteBank16],offset ACUMOS_bank
        mov     [WriteBank16Len],ACUMOS_bank_len
        mov     [WriteBank256],offset ACUMOS_bank
        mov     [WriteBank256Len],ACUMOS_bank_len
        mov     [SetupSVGA],offset ACUMOS_setup
        mov     [SetupSVGALen],ACUMOS_setup_len
        mov     [ExitSVGA],offset ACUMOS_exit
        mov     [ExitSVGALen],ACUMOS_exit_len
        ret

ENDP    ACUMOS_init
